/* This code was taken directly from ALSA's aplay/arecord tool */

#include "alsa.h"
int alsa_init()
{
	char *pcm_name = "plug:0,0";
	int err;
	snd_pcm_info_t *info;

	err = snd_output_stdio_attach(&log, stderr, 0);
	assert(err >= 0);

	file_type = FORMAT_DEFAULT;
		stream = SND_PCM_STREAM_CAPTURE;
		file_type = FORMAT_WAVE;
		command = "arecord";

	chunk_size = -1;
	rhwparams.format = SND_PCM_FORMAT_U8;
	rhwparams.rate = DEFAULT_SPEED;
	rhwparams.channels = 1;
	memset(&spdif, 0, sizeof(spdif));


	err = snd_pcm_open(&handle, pcm_name, stream, open_mode);
	if (err < 0) {
		error("audio open error: %s", snd_strerror(err));
		return 1;
	}

	snd_pcm_info_alloca(&info);
	if ((err = snd_pcm_info(handle, info)) < 0) {
		error("info error: %s", snd_strerror(err));
		return 1;
	}

	if (digtype != SND_CTL_ELEM_TYPE_NONE) {
		snd_ctl_elem_value_t *ctl;
		snd_ctl_t *ctl_handle;
		char ctl_name[12];
		int ctl_card;
		snd_ctl_elem_value_alloca(&ctl);
		snd_ctl_elem_value_set_interface(ctl, SND_CTL_ELEM_IFACE_PCM);
		snd_ctl_elem_value_set_device(ctl, snd_pcm_info_get_device(info));
		snd_ctl_elem_value_set_subdevice(ctl, snd_pcm_info_get_subdevice(info));
		snd_ctl_elem_value_set_name(ctl, SND_CTL_NAME_IEC958("",PLAYBACK,PCM_STREAM));
		snd_ctl_elem_value_set_iec958(ctl, &spdif);
		ctl_card = snd_pcm_info_get_card(info);
		if (ctl_card < 0) {
			error("Unable to setup the IEC958 (S/PDIF) interface - PCM has no assigned card");
			goto __diga_end;
		}
		sprintf(ctl_name, "hw:%d", ctl_card);
		if ((err = snd_ctl_open(&ctl_handle, ctl_name)) < 0) {
			error("Unable to open the control interface '%s': %s", ctl_name, snd_strerror(err));
			goto __diga_end;
		}
		if ((err = snd_ctl_elem_write(ctl_handle, ctl)) < 0) {
			error("Unable to update the IEC958 control: %s", snd_strerror(err));
			goto __diga_end;
		}
		snd_ctl_close(ctl_handle);
	      __diga_end:
	}

	if (nonblock) {
		err = snd_pcm_nonblock(handle, 1);
		if (err < 0) {
			error("nonblock setting error: %s", snd_strerror(err));
			return 1;
		}
	}

	chunk_size = 1024;
	hwparams = rhwparams;

	audiobuf = (char *)malloc(1024);
	if (audiobuf == NULL) {
		error("not enough memory");
		return 1;
	}

	if (mmap_flag) {
		writei_func = snd_pcm_mmap_writei;
		readi_func = snd_pcm_mmap_readi;
		writen_func = snd_pcm_mmap_writen;
		readn_func = snd_pcm_mmap_readn;
	} else {
		writei_func = snd_pcm_writei;
		readi_func = snd_pcm_readi;
		writen_func = snd_pcm_writen;
		readn_func = snd_pcm_readn;
	}
capture(NULL);

return(1);
}

void alsa_close(){	
	
	snd_pcm_close(handle);
	free(audiobuf);
	snd_output_close(log);
}

ssize_t safe_read(int fd, void *buf, size_t count)
{
	ssize_t result = 0, res;

	while (count > 0) {
		if ((res = read(fd, buf, count)) == 0)
			break;
		if (res < 0)
			return result > 0 ? result : res;
		count -= res;
		result += res;
		(char *)buf += res;
	}
	return result;
}


static void set_params(void)
{
	snd_pcm_hw_params_t *params;
	snd_pcm_sw_params_t *swparams;
	size_t buffer_size;
	int err;
	size_t n;
	size_t xfer_align;
	snd_pcm_hw_params_alloca(&params);
	snd_pcm_sw_params_alloca(&swparams);
	err = snd_pcm_hw_params_any(handle, params);
	if (err < 0) {
		error("Broken configuration for this PCM: no configurations available");
		exit(EXIT_FAILURE);
	}
	if (mmap_flag) {
		snd_pcm_access_mask_t *mask = alloca(snd_pcm_access_mask_sizeof());
		snd_pcm_access_mask_none(mask);
		snd_pcm_access_mask_set(mask, SND_PCM_ACCESS_MMAP_INTERLEAVED);
		snd_pcm_access_mask_set(mask, SND_PCM_ACCESS_MMAP_NONINTERLEAVED);
		snd_pcm_access_mask_set(mask, SND_PCM_ACCESS_MMAP_COMPLEX);
		err = snd_pcm_hw_params_set_access_mask(handle, params, mask);
	} else if (interleaved)
		err = snd_pcm_hw_params_set_access(handle, params,
						   SND_PCM_ACCESS_RW_INTERLEAVED);
	else
		err = snd_pcm_hw_params_set_access(handle, params,
						   SND_PCM_ACCESS_RW_NONINTERLEAVED);
	if (err < 0) {
		error("Access type not available");
		exit(EXIT_FAILURE);
	}
	err = snd_pcm_hw_params_set_format(handle, params, hwparams.format);
	if (err < 0) {
		error("Sample format non available");
		exit(EXIT_FAILURE);
	}
	err = snd_pcm_hw_params_set_channels(handle, params, hwparams.channels);
	if (err < 0) {
		error("Channels count non available");
		exit(EXIT_FAILURE);
	}

#if 0
	err = snd_pcm_hw_params_set_periods_min(handle, params, 2);
	assert(err >= 0);
#endif
	err = snd_pcm_hw_params_set_rate_near(handle, params, hwparams.rate, 0);
	assert(err >= 0);
	if (buffer_time < 0)
		buffer_time = 500000;
	buffer_time = snd_pcm_hw_params_set_buffer_time_near(handle, params,
							     buffer_time, 0);
	assert(buffer_time >= 0);
	if (period_time < 0)
		period_time = buffer_time / 4;
	period_time = snd_pcm_hw_params_set_period_time_near(handle, params,
							     period_time, 0);
	assert(period_time >= 0);
	err = snd_pcm_hw_params(handle, params);
	if (err < 0) {
		error("Unable to install hw params:");
		snd_pcm_hw_params_dump(params, log);
		exit(EXIT_FAILURE);
	}
	chunk_size = snd_pcm_hw_params_get_period_size(params, 0);
	buffer_size = snd_pcm_hw_params_get_buffer_size(params);
	if (chunk_size == buffer_size) {
		error("Can't use period equal to buffer size (%u == %lu)", chunk_size, buffer_size);
		exit(EXIT_FAILURE);
	}
	snd_pcm_sw_params_current(handle, swparams);
	xfer_align = snd_pcm_sw_params_get_xfer_align(swparams);
	if (sleep_min)
		xfer_align = 1;
	err = snd_pcm_sw_params_set_sleep_min(handle, swparams,
					      sleep_min);
	assert(err >= 0);
	if (avail_min < 0)
		n = chunk_size;
	else
		n = snd_pcm_hw_params_get_rate(params, 0) * (double) avail_min / 1000000;
	err = snd_pcm_sw_params_set_avail_min(handle, swparams, n);
				   
	err = snd_pcm_sw_params_set_xfer_align(handle, swparams, xfer_align);
	assert(err >= 0);
	if (snd_pcm_sw_params(handle, swparams) < 0) {
		error("unable to install sw params:");
		snd_pcm_sw_params_dump(swparams, log);
		exit(EXIT_FAILURE);
	}

	if (verbose)
		snd_pcm_dump(handle, log);

	bits_per_sample = snd_pcm_format_physical_width(hwparams.format);
	bits_per_frame = bits_per_sample * hwparams.channels;
	chunk_bytes = chunk_size * bits_per_frame / 8;
	audiobuf = realloc(audiobuf, chunk_bytes);
	if (audiobuf == NULL) {
		error("not enough memory");
		exit(EXIT_FAILURE);
	}
}

#ifndef timersub
#define	timersub(a, b, result) \
do { \
	(result)->tv_sec = (a)->tv_sec - (b)->tv_sec; \
	(result)->tv_usec = (a)->tv_usec - (b)->tv_usec; \
	if ((result)->tv_usec < 0) { \
		--(result)->tv_sec; \
		(result)->tv_usec += 1000000; \
	} \
} while (0)
#endif

void xrun(void)
{
//	snd_pcm_status_t *status;
//	int res;
//	
//	snd_pcm_status_alloca(&status);
//	if ((res = snd_pcm_status(handle, status))<0) {
//		error("status error: %s", snd_strerror(res));
//		exit(EXIT_FAILURE);
//	}
//	if (snd_pcm_status_get_state(status) == SND_PCM_STATE_XRUN) {
//		struct timeval now, diff, tstamp;
//		gettimeofday(&now, 0);
//		snd_pcm_status_get_trigger_tstamp(status, &tstamp);
//		timersub(&now, &tstamp, &diff);
//		fprintf(stderr, "xrun!!! (at least %.3f ms long)\n", diff.tv_sec * 1000 + diff.tv_usec / 1000.0);
//		if (verbose) {
//			fprintf(stderr, "Status:\n");
//			snd_pcm_status_dump(status, log);
//		}
//		res = snd_pcm_prepare(handle);
//			error("xrun: prepare error: %s", snd_strerror(res));
//			exit(EXIT_FAILURE);
//		}
//		return;		/* ok, data should be accepted again */
//	}
//	error("read/write error");
//	exit(EXIT_FAILURE);
}


static ssize_t pcm_read(u_char *data, size_t rcount)
{
	ssize_t r;
	size_t result = 0;
	size_t count = rcount;
	int res;

	if (sleep_min == 0 &&
	    count != chunk_size) {
		count = chunk_size;
	}

	while (count > 0) {
		r = readi_func(handle, data, count);
		if (r == -EAGAIN || (r >= 0 && r < count)) {
			snd_pcm_wait(handle, 1000);
		} else if (r == -EPIPE) {
			res = snd_pcm_prepare(handle);
			fprintf(stderr, "audio buffer overrun!\n");
//			xrun();
		} else if (r < 0) {
			error("read error: %s", snd_strerror(r));
			exit(EXIT_FAILURE);
		}
		if (r > 0) {
			result += r;
			count -= r;
			data += r * bits_per_frame / 8;
		}
	}
	return rcount;
}

void capture_go(int fd, size_t count, int rtype, char *name)
{
	set_params();
}

static void capture(char *name)
{
	count = 1024;
	count += count % 2;
	capture_go(1, count, file_type, name);
}

void acapture(){
	
	size_t c;
	ssize_t r;
	int count;
		
	count=1024;
	
	while (count > 0) {
		c = count;
		if (c > chunk_bytes)
			c = chunk_bytes;
		c = c * 8 / bits_per_frame;
		r = pcm_read(sample_buffer, c);
		r = r * bits_per_frame / 8;
		count -= r;
	}
}